/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.adapter

import android.content.Context
import android.content.res.Configuration
import android.graphics.Bitmap
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.RecyclerView
import com.kakao.sdk.partner.friend.imageloader.ImageLoader
import com.kakao.sdk.partner.friend.internal.ChatItem
import com.kakao.sdk.partner.friend.internal.Item
import com.kakao.sdk.partner.friend.model.PickerChatType
import com.kakao.sdk.partner.friend.util.ChatProfileImageMaker
import com.kakao.sdk.partner.friend.util.LayoutUtils
import com.kakao.sdk.partner.friend.util.getDefaultProfileImageBitmap
import com.kakao.sdk.v2.partner.friend.R
import com.kakao.sdk.v2.partner.friend.databinding.KakaoSdkCategoryDividerBinding
import com.kakao.sdk.v2.partner.friend.databinding.KakaoSdkEmptySearchResultTextBinding
import com.kakao.sdk.v2.partner.friend.databinding.KakaoSdkHeaderFriendPickerBinding
import com.kakao.sdk.v2.partner.friend.databinding.KakaoSdkItemChatPickerBinding
import kotlin.math.min
import kotlin.random.Random

/**
 * @suppress
 */
internal class ChatsListAdapter(
    private val isPopup: Boolean,
    private val checkCallback: (ChatItem.Chat) -> Unit,
    private val uncheckCallback: (ChatItem.Chat) -> Unit
) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    private var items = mutableListOf<Item>()
    private lateinit var mRecyclerView: RecyclerView
    private val diffUtilCallback = ItemDiffUtilCallback()
    private var pickedChat: ChatItem.Chat? = null

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        val inflater = LayoutInflater.from(parent.context)
        return when (viewType) {
            VIEW_TYPE_HEADER -> {
                HeaderViewHolder(
                    KakaoSdkHeaderFriendPickerBinding.inflate(
                        inflater,
                        parent,
                        false
                    )
                )
            }
            VIEW_TYPE_CHAT -> {
                ChatViewHolder(
                    isPopup,
                    KakaoSdkItemChatPickerBinding.inflate(inflater, parent, false),
                    checkCallback = {
                        pickedChat?.let { prevChat ->
                            uncheckCallback(prevChat)
                        }
                        checkCallback(it)
                    }
                ) {
                    uncheckCallback(it)
                }
            }
            VIEW_CATEGORY_DIVIDER -> {
                DividerViewHolder(
                    KakaoSdkCategoryDividerBinding.inflate(
                        inflater,
                        parent,
                        false
                    )
                )
            }
            VIEW_EMPTY_SEARCH_RESULT_TEXT -> {
                EmptySearchViewHolder(
                    isPopup,
                    KakaoSdkEmptySearchResultTextBinding.inflate(inflater, parent, false)
                )
            }
            else -> throw ClassCastException("Unknown viewType $viewType")
        }
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        val item = items[position]
        when (holder) {
            is HeaderViewHolder -> holder.bind(item as Item.Header)
            is ChatViewHolder -> holder.bind(item as ChatItem.Chat)
            is EmptySearchViewHolder -> holder.setLayout()
        }
    }

    override fun getItemViewType(position: Int): Int {
        return when (items[position]) {
            is Item.Header -> VIEW_TYPE_HEADER
            is ChatItem.Chat -> VIEW_TYPE_CHAT
            is Item.Divider -> VIEW_CATEGORY_DIVIDER
            is Item.EmptySearchResult -> VIEW_EMPTY_SEARCH_RESULT_TEXT
            else -> throw ClassCastException("Unknown viewType ${items[position]::class.java}")
        }
    }

    override fun onAttachedToRecyclerView(recyclerView: RecyclerView) {
        super.onAttachedToRecyclerView(recyclerView)
        mRecyclerView = recyclerView
    }

    override fun getItemCount(): Int = items.size

    fun checkChat(chat: ChatItem.Chat) {
        updateChat(chat, true)
    }

    fun uncheckChat() {
        pickedChat?.let {
            updateChat(it, false)
        }
    }

    fun setPickedChat(chat: ChatItem.Chat?) {
        this.pickedChat = chat
    }

    private fun updateChat(chat: ChatItem.Chat, isChecked: Boolean) {
        val index = items.indexOf(chat)
        // index를 못찾은 경우에는 -1 이므로 crash 방지 (검색 중에는 리스트에 해당 chat이 없을 수 있음)
        if (index > -1) {
            (items[index] as ChatItem.Chat).isChecked = isChecked
        }
        notifyItemChanged(index)
    }

    fun setList(items: MutableList<Item>) {
        diffUtilCallback.setLists(this.items, items)
        DiffUtil.calculateDiff(diffUtilCallback).dispatchUpdatesTo(this)
        this.items = items
    }

    class HeaderViewHolder(
        private val binding: KakaoSdkHeaderFriendPickerBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        fun bind(header: Item.Header) {
            binding.titleTv.text = header.title
        }
    }

    class DividerViewHolder(binding: KakaoSdkCategoryDividerBinding) :
        RecyclerView.ViewHolder(binding.root)

    class EmptySearchViewHolder(
        private val isPopup: Boolean,
        private val binding: KakaoSdkEmptySearchResultTextBinding
    ) : RecyclerView.ViewHolder(binding.root) {
        fun setLayout() {
            binding.apply {
                if (LayoutUtils.orientation == Configuration.ORIENTATION_PORTRAIT) {
                    if (isPopup) {
                        emptySearchResultText.apply {
                            (layoutParams as LinearLayout.LayoutParams).topMargin =
                                resources.getDimensionPixelSize(R.dimen.popup_empty_search_result_margin_top)
                        }
                    }
                } else {
                    emptySearchResultText.apply {
                        (layoutParams as LinearLayout.LayoutParams).topMargin = if (isPopup) {
                            resources.getDimensionPixelSize(R.dimen.popup_empty_search_result_margin_top_landscape)
                        } else {
                            resources.getDimensionPixelSize(R.dimen.empty_search_result_margin_top_landscape)
                        }
                    }
                }
            }
        }
    }

    class ChatViewHolder(
        private val isPopup: Boolean,
        private val binding: KakaoSdkItemChatPickerBinding,
        private val checkCallback: (ChatItem.Chat) -> Unit,
        private val uncheckCallback: (ChatItem.Chat) -> Unit
    ) : RecyclerView.ViewHolder(binding.root) {

        fun bind(chat: ChatItem.Chat) {
            binding.apply {
                root.setOnClickListener {
                    if (!checkBox.isChecked) {
                        checkCallback(chat)
                    } else {
                        uncheckCallback(chat)
                    }
                }

                setCheckbox(chat)
                titleTv.text = chat.title
                chat.memberCount?.let {
                    memberCountTv.text = it.toString()
                    memberCountTv.visibility = if (it < 3) View.GONE else View.VISIBLE
                }
                setOpenChatIcon(binding.root.context, chat, openChatImage)
                setChatRoomImage(chat)
            }
        }

        private fun setOpenChatIcon(context: Context, chat: ChatItem.Chat, openChatIconView: View) {
            chat.chatType?.let {

                val startMargin = if (it == PickerChatType.OPEN) {
                    openChatIconView.visibility = View.VISIBLE
                    if (isPopup) {
                        context.resources.getDimensionPixelSize(R.dimen.popup_list_item_open_chat_title_margin_start)
                    } else {
                        context.resources.getDimensionPixelSize(R.dimen.list_item_open_chat_title_margin_start)
                    }
                } else {
                    openChatIconView.visibility = View.GONE
                    if (isPopup) {
                        context.resources.getDimensionPixelSize(R.dimen.popup_list_item_nickname_margin_start)
                    } else {
                        context.resources.getDimensionPixelSize(R.dimen.list_item_nickname_margin_start)
                    }
                }

                (binding.titleTv.layoutParams as? ConstraintLayout.LayoutParams)?.let {
                    it.marginStart = startMargin
                }
            }
        }

        private fun setChatRoomImage(chat: ChatItem.Chat) {
            binding.apply {
                val cacheProfileImage = ChatProfileImageMaker.getCacheImage(root.context, chat.id)
                if (cacheProfileImage != null) {
                    profileIv.setImageBitmap(cacheProfileImage)
                    return
                }

                val urls = mutableListOf<String>()

                if (chat.imageUrl.isNullOrEmpty()) {
                    if (chat.displayMemberImages.isNullOrEmpty()) {
                        // 모든 chatMember 가 default profile
                        chat.memberCount?.let {
                            val image = makeDefaultProfileImageOnly(root.context, chat)
                            profileIv.setImageBitmap(image)
                            return
                        }
                    } else {
                        urls.addAll(chat.displayMemberImages.take(4))
                    }
                } else {
                    urls.add(chat.imageUrl)
                    ImageLoader.get(root.context).load(urls).toBitmap { bitmap ->
                        val profileImages = mutableListOf(bitmap)
                        val image = ChatProfileImageMaker.makeProfileImage(
                            context = root.context,
                            id = chat.id,
                            profileImages = profileImages
                        )
                        profileIv.setImageBitmap(image)
                    }
                    return
                }

                ImageLoader.get(context = root.context).load(urls).toMultiBitmap { bitmaps ->
                    val bitmap = if (bitmaps != null) {
                        val profileImages = bitmaps.toMutableList()
                        val count = getExtraDefaultProfileImageCount(chat, bitmaps.size)
                        repeat(count) {
                            makeDefaultImage(root.context)?.let {
                                profileImages.add(it)
                            }
                        }

                        ChatProfileImageMaker.makeProfileImage(
                            context = root.context,
                            id = chat.id,
                            profileImages = profileImages.toMutableList()
                        )
                    } else {
                        makeDefaultProfileImageOnly(root.context, chat)
                    }
                    profileIv.setImageBitmap(bitmap)
                }
            }
        }

        private fun makeDefaultImage(context: Context): Bitmap? {
            return getDefaultProfileImageBitmap(
                Random.nextInt(),
                context,
                ChatProfileImageMaker.width,
                ChatProfileImageMaker.height
            )
        }

        private fun setCheckbox(chat: ChatItem.Chat) {
            binding.checkBox.apply {
                val checkBoxDrawable = R.drawable.kakao_sdk_daynight_radio
                setButtonDrawable(checkBoxDrawable)

                isChecked = chat.isChecked
            }
        }

        private fun makeDefaultProfileImageOnly(
            context: Context,
            chat: ChatItem.Chat
        ): Bitmap {
            val images = mutableListOf<Bitmap?>()
            val count = getExtraDefaultProfileImageCount(chat, 0)
            repeat(count) {
                images.add(makeDefaultImage(context))
            }

            return ChatProfileImageMaker.makeProfileImage(
                context = context,
                id = chat.id,
                profileImages = images
            )
        }

        private fun getExtraDefaultProfileImageCount(chat: ChatItem.Chat, imageCount: Int): Int {
            var extraDefaultProfileCount = 0
            chat.memberCount?.let {
                val maxChatProfile = min(it - 1, 4)
                if (maxChatProfile > imageCount) {
                    extraDefaultProfileCount = maxChatProfile - imageCount
                }
            }
            return extraDefaultProfileCount
        }
    }

    companion object {
        const val VIEW_TYPE_HEADER = 0
        const val VIEW_TYPE_CHAT = 1
        const val VIEW_CATEGORY_DIVIDER = 2
        const val VIEW_EMPTY_SEARCH_RESULT_TEXT = 3
    }
}