/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.imageloader

import android.content.Context
import android.graphics.Bitmap
import android.os.Handler
import android.os.Looper
import android.os.Message
import android.util.LruCache
import android.widget.ImageView
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.friend.imageloader.model.DecodingResult
import com.kakao.sdk.partner.friend.imageloader.model.DownloadResult
import com.kakao.sdk.partner.friend.imageloader.model.MultiDecodingResult
import com.kakao.sdk.partner.friend.imageloader.network.ConvertBitmapAction
import com.kakao.sdk.partner.friend.imageloader.network.ConvertMultiBitmapAction
import com.kakao.sdk.partner.friend.imageloader.network.ImageViewAction
import java.io.File
import java.util.concurrent.ExecutorService
import java.util.concurrent.Executors

/**
 * @suppress
 */
internal class ImageLoader {
    private val service: ExecutorService = Executors.newCachedThreadPool()
    private var downloader = ImageDownloader(mainThreadHandler)

    fun load(imageUrl: String?): ActionCreator {
        return ActionCreator(this, listOf(imageUrl), downloader)
    }

    fun load(imageUrls: List<String?>): ActionCreator {
        return ActionCreator(this, imageUrls, downloader)
    }

    fun submit(action: ImageViewAction) {
        service.submit(action)
    }

    fun submit(action: ConvertBitmapAction) {
        service.submit(action)
    }

    fun submit(action: ConvertMultiBitmapAction) {
        service.submit(action)
    }

    companion object {
        internal lateinit var diskCache: File
        private lateinit var memoryCache: LruCache<String, Bitmap>
        private const val DISK_CACHE = "image-loader-cache"

        private val instance by lazy {
            ImageLoader()
        }

        @JvmStatic
        fun get(context: Context): ImageLoader {
            if (!::memoryCache.isInitialized) {
                memoryCache = LruCache(calculateMemoryCacheSize(context))
            }
            if (!::diskCache.isInitialized) {
                diskCache = File(context.applicationContext.cacheDir, DISK_CACHE)
                if (!diskCache.exists()) {
                    diskCache.mkdirs()
                }
            }
            return instance
        }

        @JvmStatic
        internal fun loadCache(imageUrl: String): Bitmap? = memoryCache.get(imageUrl)

        @JvmStatic
        internal fun saveCache(imageUrl: String, bitmap: Bitmap) = memoryCache.put(imageUrl, bitmap)

        private val mainThreadHandler: Handler = object : Handler(Looper.getMainLooper()) {
            override fun handleMessage(msg: Message) {
                when (msg.what) {
                    Constants.IMAGE_DOWNLOAD_COMPLETE -> {
                        when (val result = msg.obj) {
                            is DownloadResult -> {
                                drawBitmap(result.target, result.bitmap)
                            }
                            is DecodingResult -> {
                                getBitmap(result.bitmap, result.callback)
                            }
                            is MultiDecodingResult -> {
                                getBitmap(result.bitmaps, result.callback)
                            }
                        }
                    }
                    Constants.IMAGE_DOWNLOAD_FAIL -> {
                        when (val result = msg.obj) {
                            is DownloadResult -> {
                                SdkLog.e("Image Loading failed: ${result.error}")
                            }
                            is DecodingResult -> {
                                SdkLog.e("Image Loading failed: ${result.error}")
                                getBitmap(null, result.callback)
                            }
                            is MultiDecodingResult -> {
                                SdkLog.e("Image Loading failed: ${result.error}")
                                getBitmap(null, result.callback)
                            }
                        }
                    }
                }
            }
        }

        @JvmStatic
        fun drawBitmap(target: ImageView?, bitmap: Bitmap?) {
            if (target == null) throw NullPointerException("target view is null")
            bitmap?.let {
                target.setImageBitmap(bitmap)
            }
        }

        @JvmStatic
        fun getBitmap(bitmap: Bitmap?, callback: (bitmap: Bitmap?) -> Unit) {
            callback(bitmap)
        }

        @JvmStatic
        fun getBitmap(bitmaps: List<Bitmap>?, callback: (bitmaps: List<Bitmap>?) -> Unit) {
            callback(bitmaps)
        }
    }
}