/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.model

import android.os.Parcelable
import com.kakao.sdk.partner.friend.Constants
import kotlinx.parcelize.Parcelize

/**
 * @property title 피커의 타이틀 영역에 표시될 텍스트
 * @property viewAppearance 피커 화면 모드
 * @property orientation 피커의 방향
 * @property enableSearch 검색 기능 사용 여부
 * @property disableSelectOptions 친구 목록이나 채팅방 멤버 목록에서 사용자 선택 불가 처리
 * @property displayAllProfile 앱과 연결되지 않았거나 동의 항목에 미동의한 사용자의 프로필 전체 정보 표시 여부
 * @property maxPickableCount 선택 가능한 친구 수의 최대값
 * @property minPickableCount 선택 가능한 친구 수의 최소값
 * @property friendsParams 탭 피커에서 친구 목록 호출 시 전달할 추가 파라미터
 * @property chatParams 탭 피커에서 채팅방 목록 호출 시 전달할 추가 파라미터
 */
@Parcelize
class PickerTabRequestParams @JvmOverloads constructor(
    val title: String? = null,
    val viewAppearance: ViewAppearance? = Constants.DEFAULT_PICKER_VIEW_APPEARANCE,
    val orientation: PickerOrientation? = Constants.DEFAULT_PICKER_ORIENTATION,
    val enableSearch: Boolean? = Constants.DEFAULT_ENABLE_SEARCH,
    val disableSelectOptions: List<DisableSelectOption>? = null,
    val displayAllProfile: Boolean? = Constants.DEFAULT_DISPLAY_ALL_PROFILE,
    val maxPickableCount: Int? = Constants.DEFAULT_MAX_PICKABLE_COUNT,
    val minPickableCount: Int? = Constants.DEFAULT_MIN_PICKABLE_COUNT,
    val friendsParams: FriendsRequestParams,
    val chatParams: ChatRequestParams
) : Parcelable {
    /**
     * @property serviceTypeFilter 친구를 가져올 서비스
     * @property friendFilter 친구 목록 필터링 기준
     * @property countryCodeFilters 카카오톡에 등록된 전화번호의 국가 코드를 토대로 요청 결과를 필터링할 때 사용 (카카오톡에 연결되지 않고 카카오스토리만 연결된 친구의 경우 미적용)
     * @property usingOsFilter 카카오톡 가입 기기의 OS 정보를 토대로 요청 결과를 필터링할 때 사용 (카카오톡에 연결되지 않고 카카오스토리만 연결된 친구의 경우 미적용)
     * @property friendOrder 친구 목록 정렬 기준
     * @property enableIndex 친구 목록 인덱스뷰 사용 여부
     * @property showMyProfile 내 프로필 표시 여부
     * @property showFavorite 즐겨찾기 친구 표시 여부
     * @property showPickedFriend 선택한 친구 표시 여부
     */
    @Parcelize
    class FriendsRequestParams @JvmOverloads constructor(
        val serviceTypeFilter: PickerServiceTypeFilter? = Constants.DEFAULT_PICKER_SERVICE_TYPE_FILTER,
        val friendFilter: PickerFriendFilter? = Constants.DEFAULT_PICKER_FRIEND_FILTER,
        val countryCodeFilters: List<String>? = null,
        val usingOsFilter: PickerUsingOsFilter? = Constants.DEFAULT_OS_FILTER,
        val friendOrder: PickerFriendOrder? = null,
        val enableIndex: Boolean? = Constants.DEFAULT_ENABLE_INDEX,
        val showMyProfile: Boolean? = Constants.DEFAULT_SHOW_MY_PROFILE,
        val showFavorite: Boolean? = Constants.DEFAULT_SHOW_FAVORITE,
        val showPickedFriend: Boolean? = Constants.DEFAULT_SHOW_PICKED_FRIENDS
    ) : Parcelable

    /**
     * @property selectionType 채팅방 피커 종류, CHAT 또는 CHAT_MEMBER 선택(필수)
     * @property chatFilters 채팅방 피커에 포함할 채팅방 타입
     */
    @Parcelize
    class ChatRequestParams @JvmOverloads constructor(
        val selectionType: PickerChatSelectionType,
        val chatFilters: List<PickerChatFilter>? = listOf(Constants.DEFAULT_PICKER_CHAT_FILTER)
    ) : Parcelable
}