/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.view

import android.app.Activity
import android.content.pm.ActivityInfo
import android.os.Bundle
import android.os.Parcelable
import android.os.ResultReceiver
import androidx.appcompat.app.AppCompatActivity
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.friend.Constants
import com.kakao.sdk.partner.friend.Repository
import com.kakao.sdk.partner.friend.model.PickerOrientation
import com.kakao.sdk.partner.friend.model.ViewAppearance
import com.kakao.sdk.partner.friend.internal.ViewSettingParams
import com.kakao.sdk.partner.friend.util.LayoutUtils
import com.kakao.sdk.v2.partner.friend.R

/**
 * @suppress
 */
internal abstract class BaseActivity : AppCompatActivity() {
    private lateinit var resultReceiver: ResultReceiver

    private fun setViewAppearance(isPopup: Boolean, param: ViewSettingParams?) {
        if (param?.viewAppearance == ViewAppearance.LIGHT) {
            setTheme(if (isPopup) R.style.DayOnlyPopup else R.style.DayOnlyFull)
        } else if (param?.viewAppearance == ViewAppearance.DARK) {
            setTheme(if (isPopup) R.style.NightOnlyPopup else R.style.NightOnlyFull)
        }
    }

    private fun setViewOrientation(param: ViewSettingParams?) {
        if (param?.orientation == PickerOrientation.PORTRAIT) {
            requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_SENSOR_PORTRAIT
        } else if (param?.orientation == PickerOrientation.LANDSCAPE) {
            requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_SENSOR_LANDSCAPE
        }
    }

    private fun setFragment(pickerType: String) {
        val fragment = when (pickerType) {
            Constants.FRIEND_PICKER -> FriendPickerFragment::class.java
            Constants.CHAT_PICKER -> ChatPickerFragment::class.java
            else -> TabFragment::class.java
        }
        supportFragmentManager.beginTransaction()
            .add(R.id.fragment_container, fragment.newInstance()).commit()
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        try {
            intent.extras?.getBundle(Constants.KEY_BUNDLE)?.run {
                resultReceiver =
                    getParcelable<ResultReceiver>(Constants.KEY_RESULT_RECEIVER) as ResultReceiver
                val viewSettingParam =
                    getParcelable<Parcelable>(Constants.KEY_VIEW_SETTING_PARAM) as? ViewSettingParams
                val isPopup = getBoolean(Constants.KEY_IS_POPUP)
                val pickerType = getString(Constants.KEY_PICKER_TYPE) ?: ""

                setViewAppearance(isPopup, viewSettingParam)
                setViewOrientation(viewSettingParam)
                LayoutUtils.initialize(applicationContext, isPopup)

                if (savedInstanceState == null) {
                    setFragment(pickerType)
                }
            }
        } catch (e: Throwable) {
            SdkLog.e(e)
            sendError(ClientError(ClientErrorCause.Unknown).apply { initCause(e) })
        }
    }

    override fun onBackPressed() {
        if (supportFragmentManager.backStackEntryCount == 0) {
            sendError(ClientError(ClientErrorCause.Cancelled, "cancelled."))
            Repository.clearData()
        }
        super.onBackPressed()
    }

    private fun sendError(exception: KakaoSdkError) {
        if (this::resultReceiver.isInitialized) {
            resultReceiver.send(
                Activity.RESULT_CANCELED,
                Bundle().apply {
                    putSerializable(Constants.KEY_EXCEPTION, exception)
                }
            )
        }
        finish()
    }
}