/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.viewmodel

import android.app.Activity
import android.content.Context
import android.os.Bundle
import android.os.ResultReceiver
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import com.kakao.sdk.partner.friend.Constants
import com.kakao.sdk.partner.friend.Repository
import com.kakao.sdk.partner.friend.internal.ChatItem
import com.kakao.sdk.partner.friend.internal.Item
import com.kakao.sdk.partner.friend.model.PickerChatRequestParams
import com.kakao.sdk.partner.friend.model.SelectedChat
import com.kakao.sdk.partner.friend.network.model.PickerChat
import com.kakao.sdk.partner.friend.util.PhonemeUtils
import kotlin.properties.Delegates

/**
 * @suppress
 */
internal class ChatViewModelFactory(
    private val isPopup: Boolean,
    private val isTab: Boolean,
    private val resultReceiver: ResultReceiver,
    private val param: PickerChatRequestParams
) : ViewModelProvider.Factory {
    override fun <T : ViewModel?> create(modelClass: Class<T>): T {
        return if (modelClass.isAssignableFrom(ChatViewModel::class.java)) {
            ChatViewModel(isPopup, isTab, resultReceiver, param) as T
        } else {
            throw IllegalArgumentException()
        }
    }
}

/**
 * @suppress
 */
internal class ChatViewModel(
    val isPopup: Boolean,
    val isTab: Boolean,
    val resultReceiver: ResultReceiver,
    val param: PickerChatRequestParams
) : ViewModel() {
    private val totalList = mutableListOf<Item>()

    private val _chatList = mutableListOf<ChatItem.Chat>()

    private val _pickerList = MutableLiveData<MutableList<Item>>()
    val pickerList: LiveData<MutableList<Item>> get() = _pickerList // 리스트에 보여지는 LiveData

    private val _searchStatus = MutableLiveData<SearchStatus>().apply { value = SearchStatus.READY }
    val searchStatus: LiveData<SearchStatus> get() = _searchStatus

    private val _isError = MutableLiveData<Boolean>()
    val isError: LiveData<Boolean> get() = _isError

    private var _pickedChat = MutableLiveData<ChatItem.Chat?>()
    val pickedChat: LiveData<ChatItem.Chat?> get() = _pickedChat

    private val repository = Repository

    var displayAllProfile by Delegates.notNull<Boolean>()
        private set

    init {
        displayAllProfile = param.displayAllProfile ?: Constants.DEFAULT_DISPLAY_ALL_PROFILE

        fetchData()
    }

    // SearchBar.searchAction() 타입이 (Context, String) 이라서 파라미터에 Context를 남겨둠
    fun searchChats(context: Context, searchText: String) {
        if (searchText.isBlank()) {
            // 빈 검색창일 때
            _searchStatus.value = SearchStatus.DONE
            _searchStatus.value = SearchStatus.READY
            _pickerList.value = totalList // 전체 채팅방 리스트 보여줌
        } else {
            _searchStatus.value = SearchStatus.SEARCHING

            val filteredList = mutableListOf<Item>()

            // 검색 필터링
            filteredList.addAll(_chatList.filter {
                PhonemeUtils.isMatch(it.title ?: "", searchText)
            })

            // 검색 결과가 없을 때
            if (filteredList.size == 0) {
                filteredList.clear()
                filteredList.add(Item.EmptySearchResult)
            }
            _pickerList.value = filteredList // 필터링된 채팅방 보여줌
        }
    }

    fun addCheckedChat(chat: ChatItem.Chat) {
        _pickedChat.value = chat
        (totalList.find { it == chat } as ChatItem.Chat).isChecked = true
    }

    fun removeUncheckedChat() {
        (totalList.find { it == _pickedChat.value!! } as ChatItem.Chat).isChecked = false
        _pickedChat.value = null
    }

    fun sendResult() {
        _pickedChat.value?.let {
            resultReceiver.send(
                Activity.RESULT_OK,
                Bundle().apply {
                    putParcelable(
                        Constants.KEY_SELECTED_CHAT,
                        convertChatToResult(it)
                    )
                }
            )
        }
    }

    private fun convertChatToResult(it: ChatItem.Chat) =
        SelectedChat(it.id, it.memberCount, it.titleSource, it.imageUrl, it.chatType)

    fun loadChatMember(callback: (error: Throwable?) -> Unit) {
        pickedChat.value?.let {
            repository.loadChatMembersData(it.id) { error ->
                if (error != null) {
                    callback(error)
                } else {
                    callback(null)
                }
            }
        }
    }

    fun clearPickedChat() {
        _pickedChat.value = null
    }

    private fun fetchData() {
        repository.chats.forEach { data ->
            val chat = toItem(data)
            _chatList.add(chat)
        }

        // 친구가 한 명도 없는 경우
        if (_chatList.size == 0) {
            _pickerList.value = totalList
            _isError.value = true
            return
        }

        totalList.addAll(_chatList)
        _pickerList.value = totalList
    }

    private fun toItem(chat: PickerChat) = ChatItem.Chat(
        chat.id,
        chat.title,
        chat.titleSource,
        chat.imageUrl,
        chat.memberCount,
        chat.displayMemberImages,
        chat.chatType
    )
}